@@ -3,9 +3,9 @@ use strict;
 use warnings;
 
 package Mxpress::PDF {
-	our $VERSION = '0.05';
+	our $VERSION = '0.06';
 	use Zydeco (
-		version	=> '0.05',
+		version	=> '0.06',
 		authority => 'cpan:LNATION',
 	);
 	use Colouring::In;
@@ -293,25 +293,27 @@ package Mxpress::PDF {
 			my $page = $file ? $self->file->page : $self;
 			my $sp = $self->padding/mm;
 			my $pp = $page->padding/mm;
-			$x //= $page->x + $sp;
-			$y //= $page->y - $sp;
-			$w //= $page->w - $sp;
-			$h //= $page->oh - ($page->oh - $y) - ($sp + $pp);
-			$h = (!$page->footer->active ? $h : $h > $page->footer->h ? ($h - $page->footer->h) : ($page->saving ? $h : 0));
-			if ($file && $page->columns > 1 && !$self->full) {
-				$w = ($page->w / $page->columns);
-				$x += ($w * ($page->column - 1));
-				$w -= $pp;
-			}
-			if ($file && $page->rows > 1 && !$self->full) {
-				my $hh = $page->header->active ? $page->header->h :0;
-				my $fh = $page->footer->active ? $page->footer->h : 0;
-				$h = ($page->h + ($pp*2)) / $page->rows;
-				$h -= ((($page->h + ($pp*3)) - $y) - ($h * ($page->row - 1)));
-				$h -= $fh;
-				if ($page->row > 1 && $page->y == $page->row_y) {
-					$y -= $pp;
-					$h -= $pp;
+			$x //= ($page->x + $sp);
+			$y //= ($page->y - $sp);
+			$w //= ($page->w - $sp);
+			$h //= ($page->oh - ($page->oh - $y) - ($sp + $pp));
+			$h = (!$page->footer->active ? $h : $page->footer->saving ? ($h + $sp + $pp) : $h > $page->footer->h ? ($h - $page->footer->h) :  0);
+			if ($file) {
+				if ($page->columns > 1 && !$self->full) {
+					$w = ($page->w / $page->columns);
+					$x += ($w * ($page->column - 1));
+				}
+				$w -= ($pp + $sp);
+				if ($page->rows > 1 && !$self->full) {
+					my $hh = $page->header->active ? $page->header->h :0;
+					my $fh = $page->footer->active ? $page->footer->h : 0;
+					$h = ($page->h + ($pp*2)) / $page->rows;
+					$h -= ((($page->h + ($pp*3)) - $y) - ($h * ($page->row - 1)));
+					$h -= $fh;
+					if ($page->row > 1 && $page->y == $page->row_y) {
+						$y -= $pp;
+						$h -= $pp;
+					}
 				}
 			}
 			return $xy ? ($x, $y) : ($x, $y, $w, $h);
@@ -534,8 +536,8 @@ package Mxpress::PDF {
 				my ($fl, $fp, @paragraph) = (1, 1, split ( / /, shift(@paragraphs) || '' ));
 				# while we have enough height to add a new line
 				if ($h >= 0 && $y >= 0) {
-					while ($ypos > ($y - $h)) {
-						unless (@paragraph) {
+					while ($ypos + 0 >= ($y - $h)) {
+						unless (scalar @paragraph) {
 							last unless scalar @paragraphs;
 							@paragraph = split( / /, shift(@paragraphs) );
 							if ($page_column) {
@@ -544,8 +546,8 @@ package Mxpress::PDF {
 								$ypos = $y;
 							}
 							$ypos -= $l;
-							$ypos -= $self->paragraph_space/mm if $self->paragraph_space;
-							last unless $ypos >= $y - $h;
+							$ypos -= ($self->paragraph_space/mm) if $self->paragraph_space;
+							last unless $ypos >= ($y - $h);
 							($fl, $fp) = (1, 0);
 						}
 						my ($xpos, $lw, $line_width, @line) = ($x, $w, 0);
@@ -554,6 +556,7 @@ package Mxpress::PDF {
 							$line_width += $width{$paragraph[0]} || 0;
 							push @line, shift(@paragraph);
 						}
+
 						my ($wordspace, $align);
 						if ($self->align eq 'fulljustify' or $self->align eq 'justify' and @paragraph) {
 							if (scalar(@line) == 1) {
@@ -893,7 +896,7 @@ Mxpress::PDF - PDF
 
 =head1 VERSION
 
-Version 0.05
+Version 0.06
 
 =cut
 
